/*
 * Copyright (C) 2003 BLADOX, s.r.o.  All rights reserved.
 *
 * This file is part of an example program for Turbo. This example
 * program may be used, distributed and modified without limitation.
 *
 */

#include <config.h>
#include <turbo/turbo.h>

#include "i2cma.h"
#include "uart2i2c.h"

volatile u8 sladdr;

void init_uart2i2c (u8 myslaveaddr)
{
//__dbs("iu");__dbs("\n");
  i2c_init (myslaveaddr);
  sladdr = UART2I2C_SLAVE_ADDRESS;
}

// TRANSMIT function
u8 uart2i2c_tx_data (u8 * data, u8 len)
{
  u8 status;
  u8 i;
  u8 reps;

  reps = 0;
I2C_SEND_DATA_ERROR_TX_REP:
//__dbs("utd");__dbs("\n");
  status = i2c_send_start ();	//send START
  if (status != 0x08)
    goto I2C_SEND_DATA_ERROR_TX;	//finish with error code
  status = i2c_send_sla_w (sladdr);	//send SLA+W
  if (status != 0x18)
    goto I2C_SEND_DATA_ERROR_TX;	//finish with error code
  for (i = 0; i < len; i++)
  {
//__dbc(data[i]);
    status = i2c_send_data (data[i]);	//send DATA
    if (status != 0x28)
      goto I2C_SEND_DATA_ERROR_TX;	//finish with error code
  }
  status = i2c_send_stop ();	//send STOP
  return 0;
I2C_SEND_DATA_ERROR_TX:
  if (status == 0x20)
  {
    delayUs (100);
    reps++;
    if (reps != 100)
    {
      status = i2c_send_stop ();	//send STOP
      goto I2C_SEND_DATA_ERROR_TX_REP;	//finish with error code
    }
  }
/*
  __dbs ("utd");
  __dbs ("err=");
  __dbs ("0x");
  __dbch (status);
  __dbs ("\n");
*/
  return status;
}

// RECEIVE function
u8 uart2i2c_rx_data (u8 * data, u8 len)
{
  u8 status;
  u8 i;
  u8 reps;

  reps = 0;
I2C_SEND_DATA_ERROR_RX_REP:
//__dbs("urd");__dbs("\n");
  status = i2c_send_start ();	//send START
  if (status != 0x08)
    goto I2C_SEND_DATA_ERROR_RX;	//finish with error code
  status = i2c_send_sla_r (sladdr);	//send SLA+R
  if (status != 0x40)
    goto I2C_SEND_DATA_ERROR_RX;	//finish with error code
  for (i = 0; i < len; i++)
  {
    if (i != (len - 1))
    {				//not last byte te read, generate ACK
      status = i2c_receive_data ((u8 *) & (data[i]), 0);	//receive not last DATA
//__dbc(data[i]);
      if (status != 0x50)
	goto I2C_SEND_DATA_ERROR_RX;	//finish with error code
    }
    else
    {				//last byte te read, generate NACK
//__dbc(data[i]);
      status = i2c_receive_data (&(data[i]), 1);	//receive last DATA
      if (status != 0x58)
	goto I2C_SEND_DATA_ERROR_RX;	//finish with error code
    }
  }
  status = i2c_send_stop ();	//send STOP
  return 0;
I2C_SEND_DATA_ERROR_RX:
  if (status == 0x48)
  {
    delayUs (100);
    reps++;
    if (reps != 100)
    {
      status = i2c_send_stop ();	//send STOP
      goto I2C_SEND_DATA_ERROR_RX_REP;	//finish with error code
    }
  }
/*
  __dbs ("urd");
  __dbs ("err=");
  __dbs ("0x");
  __dbch (status);
  __dbs ("\n");
*/
  return status;
}

// SELECT register
u8 uart2i2c_select_reg (u8 reg)
{
  u8 buf[1];
  u8 status;

//__dbs("usr");__dbs("\n");
  buf[0] = reg;			//pointer reg.
  status = uart2i2c_tx_data (buf, 1);
  return status;
}

// READ rxfifo data
u8 uart2i2c_read_rxfifo_data (u8 * outbuf, u8 len)
{
  u8 status;
  u8 i;

//__dbs("urrd");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_RXFIFO_DATA);
  status = uart2i2c_rx_data (outbuf, len);
//for(i=0;i<len;i++) {__dbc(outbuf[i]); }
  return status;
}

// WRITE txfifo data
u8 uart2i2c_write_txfifo_data (u8 * inbuf, u8 len)
{
  u8 buf[64];
  u8 status;
  u8 i;

//__dbs("uwtd");__dbs("\n");
  buf[0] = UART2I2C_TXFIFO_DATA;	//pointer reg.
  for (i = 0; i < len; i++)
  {
    buf[i + 1] = inbuf[i];
  }
//for(i=0;i<len;i++) { __dbc(inbuf[i]);}
  status = uart2i2c_tx_data (buf, len + 1);
  return status;
}

// READ rxfifo status
u8 uart2i2c_read_rxfifo_status (u8 * val)
{
  u8 status;
  u8 i;

//__dbs("urrs");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_RXFIFO_STATUS);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// READ txfifo status
u8 uart2i2c_read_txfifo_status (u8 * val)
{
  u8 status;
  u8 i;

//__dbs("urts");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_TXFIFO_STATUS);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// READ rxfifo command
u8 uart2i2c_read_rxfifo_cmd (u8 * val)
{
  u8 status;
  u8 i;

//__dbs("urrc");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_RXFIFO_CMD);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// WRITE rxfifo command
u8 uart2i2c_write_rxfifo_cmd (u8 val)
{
  u8 buf[2];
  u8 status;

//__dbs("uwrc");__dbs("\n");
  buf[0] = UART2I2C_RXFIFO_CMD;	//pointer reg.
  buf[1] = val;
  status = uart2i2c_tx_data (buf, 2);
  return status;
}

// READ txfifo command
u8 uart2i2c_read_txfifo_cmd (u8 * val)
{
  u8 status;
  u8 i;

//__dbs("urtc");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_TXFIFO_CMD);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// WRITE txfifo command
u8 uart2i2c_write_txfifo_cmd (u8 val)
{
  u8 buf[2];
  u8 status;

//__dbs("uwtc");__dbs("\n");
  buf[0] = UART2I2C_TXFIFO_CMD;	//pointer reg.
  buf[1] = val;
  status = uart2i2c_tx_data (buf, 2);
  return status;
}

// READ ubrr
u8 uart2i2c_read_ubrr (u16 * val)
{
  u8 buf[2];
  u8 status;
  u8 i;

//__dbs("uru");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_UBRR);
  status = uart2i2c_rx_data (buf, 2);
  *val = (buf[1] << 8) | buf[0];
  return status;
}

// WRITE ubrr
u8 uart2i2c_write_ubrr (u16 val)
{
  u8 buf[3];
  u8 status;

//__dbs("uwu");__dbs("\n");
  buf[0] = UART2I2C_UBRR;	//pointer reg.
  buf[1] = val & 0xFF;
  buf[2] = val >> 8;
  status = uart2i2c_tx_data (buf, 3);
  return status;
}

// READ ucsr
u8 uart2i2c_read_ucsr (u8 * val)
{
  u8 status;
  u8 i;

//__dbs("uruh");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_UCSR);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// WRITE ucsr
u8 uart2i2c_write_ucsr (u8 val)
{
  u8 buf[2];
  u8 status;

//__dbs("uwuh");__dbs("\n");
  buf[0] = UART2I2C_UCSR;	//pointer reg.
  buf[1] = val;
  status = uart2i2c_tx_data (buf, 2);
  return status;
}

// READ control
u8 uart2i2c_read_control (u8 * val)
{
  u8 status;
  u8 i;

//__dbs("uruh");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_STATUS_CONTROL);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// WRITE control
u8 uart2i2c_write_control (u8 val)
{
  u8 buf[2];
  u8 status;

//__dbs("uwuh");__dbs("\n");
  buf[0] = UART2I2C_STATUS_CONTROL;	//pointer reg.
  buf[1] = val;
  status = uart2i2c_tx_data (buf, 2);
  return status;
}

// READ interval
u8 uart2i2c_read_interval (u8 * val)
{
  u8 status;
  u8 i;

//__dbs("uruh");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_ACCEL_INTERVAL);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// WRITE interval
u8 uart2i2c_write_interval (u8 val)
{
  u8 buf[2];
  u8 status;

//__dbs("uwuh");__dbs("\n");
  buf[0] = UART2I2C_ACCEL_INTERVAL;	//pointer reg.
  buf[1] = val;
  status = uart2i2c_tx_data (buf, 2);
  return status;
}

// READ rxchars
u8 uart2i2c_read_rxchars (u16 * val)
{
  u8 buf[2];
  u8 status;
  u8 i;

//__dbs("urrl");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_RXCHARS);
  status = uart2i2c_rx_data (buf, 2);
  *val = (buf[1] << 8) | buf[0];
  return status;
}

// READ txchars
u8 uart2i2c_read_txchars (u16 * val)
{
  u8 buf[2];
  u8 status;
  u8 i;

//__dbs("urtl");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_TXCHARS);
  status = uart2i2c_rx_data (buf, 2);
  *val = (buf[1] << 8) | buf[0];
  return status;
}

// READ id
u8 uart2i2c_read_id (u8 * val)
{
  u8 status;
  u8 i;

//__dbs("uri");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_ID);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// READ gps data
u8 uart2i2c_read_gps_data (u8 offs, u8 * val)
{
  u8 status;
  u8 i;

//__dbs("urgd");__dbs("\n");
  uart2i2c_select_reg (UART2I2C_GPS + offs);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// READ accelx fifo data
u8 uart2i2c_read_accelx_fifo_data (u16 * outbuf, u8 len)
{
  u8 *buf;
  u8 status;
  u8 i;

  uart2i2c_select_reg (UART2I2C_ACCELX_FIFO_DATA);
  buf = (u8 *) outbuf;
  status = uart2i2c_rx_data (buf, len * 2);
  return status;
}

// READ accely fifo data
u8 uart2i2c_read_accely_fifo_data (u16 * outbuf, u8 len)
{
  u8 *buf;
  u8 status;
  u8 i;

  uart2i2c_select_reg (UART2I2C_ACCELY_FIFO_DATA);
  buf = (u8 *) outbuf;
  status = uart2i2c_rx_data (buf, len * 2);
  return status;
}

// READ accelx words
u8 uart2i2c_read_accelx_words (u8 * val)
{
  u8 status;
  u8 i;

  uart2i2c_select_reg (UART2I2C_ACCELX_WORDS);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// READ accely words
u8 uart2i2c_read_accely_words (u8 * val)
{
  u8 status;
  u8 i;

  uart2i2c_select_reg (UART2I2C_ACCELY_WORDS);
  status = uart2i2c_rx_data (val, 1);
  return status;
}

// READ baby firmware version
u8 uart2i2c_read_baby_fw_version (u8 * outbuf, u8 len)
{
  u8 status;
  u8 i;

  uart2i2c_select_reg (UART2I2C_FIRMWARE_VERSION);
  status = uart2i2c_rx_data (outbuf, len);
  return status;
}
